/**
 * Copyright (c) 2008, Freie Universitt Berlin, Tomasz Naumowicz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Freie Universitt Berlin nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * Based on the ScatterWeb firmware from the Freie Universitaet Berlin under the BSD license.  
 */


#ifndef PROGRAMMING_H_
#define PROGRAMMING_H_

#define FILENAME_UPDATE				"UPDATE  HEX"
#define FILENAME_GOLDENIMAGE		"GOLD    HEX"

#ifndef __COMPILE_TIME
	#define __COMPILE_TIME	0
	#error "Define __COMPILE_TIME through compiler to current UTC time!"
#endif

#ifndef __APP_NAME
	#define __APP_NAME	"UNKNOWN "
	#error "Define __APP_NAME in the makefile"
#endif

#ifndef __APPVER_MINOR
	#define __APPVER_MINOR	0
	#error "Define __APPVER_MINOR in the makefile"
#endif

#ifndef __APPVER_MAJOR
	#define __APPVER_MINOR	0
	#error "Define __APPVER_MINOR in the makefile"
#endif

#ifndef __PLATFORM_BOARD
	#define __PLATFORM_BOARD		"MSB430H "
#endif

#ifndef __PLATFORM_EXTBOARD
	#define __PLATFORM_EXTBOARD		"MSB430MS"
#endif

#define APPHEADER(name, type, hwBoard, hwExtBoard, appName, appVerH, appVerL, fpReset)	\
	const appHeader_t name = {															\
		type ,																			\
		hwBoard , hwExtBoard ,															\
		appName ,{ appVerH , appVerL },													\
		(unsigned int) fpReset,															\
		__COMPILE_TIME ,																\
		( type + appVerH + appVerL ) % 0xFF												\
	}

typedef struct {					///																SIZE in bytes
	uint8_t		type;				/// type of the header											1
	uint8_t		boardName[9];		/// name of supported hardware platform (8 chars + \0)			9
	uint8_t		extBoardName[9];	/// name of supported extension board (8 chars + \0)			9
	uint8_t		appName[9];			/// name of application (8 chars + \0)							9
	uint8_t		appVer[2];			/// major and minor version of application						2
	uint16_t	resetAdr;			/// address of application entry point							2
	uint32_t	compileTime;		/// compile time in seconds since 1970-01-01 00:00:00 UTC		4
	uint8_t		checksum;			/// checksum of the header (type + appVerH + appVerL)%0xFF		1
} appHeader_t; 						///																37 bytes

/**
 * Software header. 
 * Theses variables are defined by the linker script, they are needed in the software header
 */

extern const appHeader_t appHeader;

// extern int main(void) gets the reference to the main funciton in the System.c

#define APP(type, appName, appVerH, appVerL)																			\
	extern void __attribute__((__noreturn__))   _reset_vector__(void);                      							\
	__attribute__((section(".app_header")))																				\
	APPHEADER( appHeader, type, __PLATFORM_BOARD, __PLATFORM_EXTBOARD, appName, appVerH, appVerL, &_reset_vector__ )

#define APPH_OS						0x80
#define APPH_LOADER					0x40
#define APPH_FLASHABLE				0x20
#define APPH_VERSION				0x00

#define APPH_FLAG_MASK				0xF8
#define APPH_VERSION_MASK			0x07

#define	APPH_STATIC_APP 			APPH_VERSION
#define	APPH_FLASHABLE_APP			(APPH_VERSION | APPH_FLASHABLE)

#endif /*PROGRAMMING_H_*/
