/**
 * Copyright (c) 2008, Freie Universitt Berlin, Tomasz Naumowicz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Freie Universitt Berlin nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * Based on the ScatterWeb firmware from the Freie Universitaet Berlin under the BSD license.  
 */


#ifndef SYSTEM_H_
#define SYSTEM_H_

//this defines have to be available. the USART and Timers will be configured for this value and
//will check presense of this defines
#define __MCLK_7MHZ__
#define __ACLK_32KHZ__
#define __MSP430GCC__

#define DEBUGOUTPUT                                             1

#define __INFO_START	0x1000
#define __INFO_END		0x1200
extern int __bootloader_start;
extern int __bootloader_end;

extern int  __stack;

#define __HEX_VERSION_MAJOR_ADDRESS		0x461C
#define __HEX_VERSION_MINOR_ADDRESS		0x461D

#define __HEX_COMPILETIME_ADDRESS_B1		0x4620
#define __HEX_COMPILETIME_ADDRESS_B2		0x4621
#define __HEX_COMPILETIME_ADDRESS_B3		0x4622
#define __HEX_COMPILETIME_ADDRESS_B4		0x4623

#define BOOTLD_IS_SYSTEM_LOADER			1

#define __APP_START		0x4600
//extern int __app_start;

#define STATUS_LED_ON			{ P5OUT &= ~BIT7; }
#define STATUS_LED_OFF			{ P5OUT |= BIT7; }
#define STATUS_LED_TOGGLE		{ P5OUT ^= BIT7; }

#define FATDEBUGGING					0
#define FAT_WRITE						0
#define FAT_CHANGEDIR					0
#define SD_WRITE						1
#define SD_CACHE						1
#define SD_READ_ANY						1
#define FAT_FIND_SIMPLE					1
#define	FAT_FILE_HELPER					0

#define _SHR_P4_SER			BIT0
#define _SHR_P4_RCLK		BIT1
#define _SHR_P4_SRCLR		BIT2
#define _SHR_P4_SRCLK		BIT3

#define _SHR_SRCLK_HIGH    { P4OUT |=  _SHR_P4_SRCLK; } // shift register clock (shift clock, SRCLK==SCK), the data is read at the rising edge
#define _SHR_SRCLK_LOW     { P4OUT &= ~_SHR_P4_SRCLK; }

#define _SHR_RCLK_HIGH    { P4OUT |=  _SHR_P4_RCLK; } // storage register clock (latch clock, RCLK==RCK)
#define _SHR_RCLK_LOW     { P4OUT &= ~_SHR_P4_RCLK; }

#define _SHR_SER_HIGH    { P4OUT |=  _SHR_P4_SER; } // serial input (data, SER==SER)
#define _SHR_SER_LOW     { P4OUT &= ~_SHR_P4_SER; }

#define _SHR_SRCLR_HIGH    { P4OUT |=  _SHR_P4_SRCLR; } // when low then all the outputs of all shift registers == 0
#define _SHR_SRCLR_LOW     { P4OUT &= ~_SHR_P4_SRCLR; }

typedef void(*fp_t)();

typedef int					bool;	/**< #TRUE or #FALSE */

#define	true	1					/**< TRUE */
#define false	0					/**< FALSE */


//used during development
#define VERBOSEDEBUGGING

//#include <stdarg.h>
#include <msp430x16x.h>
#include <io.h>
#include <signal.h>
//#include <stdlib.h>
//#include <stdio.h>
//#include <stdint.h>
#include <string.h>
//#include <stddef.h>
//#include <sys/types.h>

//void *memcpy(void *, const void *, size_t);
//void *memset(void *, int, size_t);


#include "Programming.h"

//////////////////////////////////////// runtime flag
//extern volatile uint8_t System_Runtime_Flag;

#define BLD_OFFLINE_STATE_SIZE					4

__attribute__((packed)) typedef struct {
  uint8_t Flashed;
  uint8_t FailStartsCounter;
  uint8_t UsingGoldenImage;
  uint8_t Checksum;
} bootloader_offline_state_t;


#include "USART.h"
#include "ScatterWeb.Sd.h"
#include "fat.h"

#if DEBUGOUTPUT
	#define LOG(x) UARTPUTC(x);
#else
	#define LOG(...)
#endif

int main(void);

#endif /*SYSTEM_H_*/
