/**
 * Copyright (c) 2008, Freie Universitt Berlin, Tomasz Naumowicz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Freie Universitt Berlin nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * Based on the ScatterWeb firmware from the Freie Universitaet Berlin under the BSD license.  
 */

#ifndef USART_H_
#define USART_H_

/**
 * @brief	Available baudrate configurations for RS232 communication
 */
enum usart_mode {
	USART_MODE_UART     = 0,
	USART_MODE_SPI      = 1
};

extern enum usart_mode		USART_CurrentMode;

#define	UART_RX					RXBUF1
#define	UART_TX					TXBUF1
#define UART_RESET_RX()			{ U1IFG &= ~URXIFG1; }
#define	UART_RESET_RXTX()		{ U1IFG &= ~(URXIFG1 | UTXIFG1); }
#define	UART_WAIT_RX()			while( (U1IFG & URXIFG1) == 0 ) { _NOP(); }
#define	UART_WAIT_TX()			while( (U1IFG & UTXIFG1) == 0 ) { _NOP(); } 
#define UART_WAIT_TXDONE()		while( (UTCTL1 & TXEPT) == 0 ) { _NOP(); }
#define USART_MODE_IS(mode) 	(USART_CurrentMode == mode)

/**
 * Initis the USART, sets the UART mode as default with the speed 115200
 */
void USART_Init();


bool USART_Lock(enum usart_mode mode);

bool USART_LockWait(enum usart_mode mode);

bool USART_Unlock();

#if DEBUGOUTPUT

void UARTTX(uint8_t* line);

void UARTPUTC(uint8_t c);

#endif

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */ 
uint8_t SPI_rx();

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */ 
void SPI_tx(const uint8_t c);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */ 
void SPI_read(void* pDestination, const uint16_t size, const bool incDest);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */ 
void SPI_write(const void* pSource, const uint16_t size, const uint8_t startToken, const bool incSource);


/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */ 
void SPI_idle(const uint16_t clocks);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */ 
uint8_t SPI_waitToken(const uint8_t token, const uint16_t timeout);


#define SPI_INIT \
	st( \
		UCTL0  = SWRST; \
		UCTL0 |= CHAR + SYNC + MM; \
		UTCTL0 = CKPH + SSEL1 + SSEL0 + STC; \
		UBR00 = 0x02; \
		UBR10 = 0x00; \
		UMCTL0 = 0x00; \
		URCTL0 = 0x00; \
		ME1 |= USPIE0; \
		UCTL0  &= ~SWRST; \
	)

//----------------------------------------------------------------------------------
//  void spiInit(uint8_t clockrate)
//
//  DESCRIPTION:
//		Init UART0 for SPI mode.
//
//  ARGUMENTS:
//      uint8_t clockrate
//			Ignore clockrate parameter (no effect for now).
//
//----------------------------------------------------------------------------------
void spi0_init(uint8_t clockrate);

#endif /*USART_H_*/
