/**
 * Copyright (c) 2008, Freie Universitt Berlin, Tomasz Naumowicz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Freie Universitt Berlin nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * Based on the ScatterWeb firmware from the Freie Universitaet Berlin under the BSD license.  
 */

#include "fat.internals.h"
#include <stdio.h>
#if FAT_FIND_SIMPLE
	bool Fat_FindFileSimple(
			const fat_state_t*		pFS,
			file_handle_t*			hParent,
			const char*				fatName,
			const uint8_t			attribMask,
			struct fat_direntry*	resultDirEntry,
			file_handle_t*  		hResultFile		// optional
	) {
		uint32_t	address = 0;
//		char		fatName[11];
	
//		Fat_CanonicalNameToFAT(fatName, name);
		
		while( Fat_GetNextDirEntry( pFS, hParent, &address, resultDirEntry ) ) {
			if( resultDirEntry->Attributes != 0 )
				if( (resultDirEntry->Attributes & attribMask) == 0 )
					continue;		
							
			if( !Fat_MatchName( resultDirEntry->Name, fatName ) )
				continue;
			
			if( hResultFile ) {
#if FATDEBUGGING
				printf("Fat_FindFileSimple: directoryentry handle address: %lu\r\n", address);
#endif
				Fat_DirEntryToHandle( hResultFile, &address, resultDirEntry ); //this address was wrong, but the resultDirEntry was OK
			}
			return true;			
		}
		return false;
	}
	
	bool Fat_MatchName(register const char* fatName, register const char* match) {
		register int c;
		
		for( c = 0; c < 11; c++, fatName++, match++ ) {
			if( *match == '\0' )		// stop matching
				break;
			else if( *match == '?' )	// don't care
				continue;
			/*
			if( match[c] == '*' ) {
				if( c > 7 ) return true;
				c = 7; continue;
			}
			*/
			else if( *match != *fatName )
				return false;
		}
		// all matched
		return true;
	}
#endif
