/*
	modified linker script for the bootloader
	this script doesn't limit the size of the bootloader since the bootloader is till under development.
	
	the linker script for the bootloader and the application have to be edited together since the linker script of the app depends on the size of the bootloader.
	this can be automated and will be done.
	
*/



OUTPUT_FORMAT("elf32-msp430","elf32-msp430","elf32-msp430")
OUTPUT_ARCH(msp:16)
MEMORY
{
  text				(rx)   	: ORIGIN = 0x2500,    LENGTH = 0xdae0
  RAM				(rwx)  	: ORIGIN = 0x1100, 	  LENGTH = 0x1400
  vectors			(rw)  	: ORIGIN = 0xffe0,    LENGTH = 32
  infomem			(rx)	: ORIGIN = 0x1000,    LENGTH = 256
  infomemnobits		(rx)	: ORIGIN = 0x1000,    LENGTH = 256
}

NOCROSSREFS(.bootloader_text .text)

SECTIONS
{
  /* space for the bootloader, not used by the application */
  .text : {
  		PROVIDE(__bootloader_start = . );
  		PROVIDE(__bootloader_header = . );
  		
	/*
	   puts the app_header of the bootloader at the beginngin of the bootloadr text.
	   this address is well defined and can be directly accessed from the application
	*/
	
	*(.app_header)
    . = ALIGN(2);

	/* bootloader text comes here, at the beginning of the flash */
    
    *(.text)
    . = ALIGN(2);
    *(.text.*)
    . = ALIGN(2);
  		
  	//. = ALIGN(0x0200);	/* make sure the end of the bootloader area ends at the end of the page (pages are 512bytes) */

	PROVIDE(__bootloader_end = . );
  		
 	PROVIDE(__application_start = . );
	PROVIDE(__application_header = . );

  }  > text
  
   _etext = .;
    
  .data   : AT (ADDR (.text) + SIZEOF (.text))
  {
     PROVIDE (__data_start = .) ;
    . = ALIGN(2);
    *(.data)
    . = ALIGN(2);
    *(.gnu.linkonce.d*)
    . = ALIGN(2);
     _edata = . ;
  }  > RAM

  /* Information memory.  */
  .infomem   :
  {
    *(.infomem)
    . = ALIGN(2);
    *(.infomem.*)
  }  > infomem
  /* Information memory (not loaded into MPU).  */
  .infomemnobits   :
  {
    *(.infomemnobits)
    . = ALIGN(2);
    *(.infomemnobits.*)
  }  > infomemnobits
  .bss  SIZEOF(.data) + ADDR(.data) :
  {
     PROVIDE (__bss_start = .) ;
    *(.bss)
    *(COMMON)
     PROVIDE (__bss_end = .) ;
     _end = . ;
  }  > RAM
  .noinit  SIZEOF(.bss) + ADDR(.bss) :
  {
     PROVIDE (__noinit_start = .) ;
    *(.noinit)
    *(COMMON)
     PROVIDE (__noinit_end = .) ;
     _end = . ;
  }  > RAM
  .vectors  :
  {
     PROVIDE (__vectors_start = .) ;
    *(.vectors*)
     _vectors_end = . ;
  }  > vectors
  
  
  /* Stabs for profiling information*/
  .profiler 0 : { *(.profiler) }
  /* Stabs debugging sections.  */
  .stab 0 : { *(.stab) }
  .stabstr 0 : { *(.stabstr) }
  .stab.excl 0 : { *(.stab.excl) }
  .stab.exclstr 0 : { *(.stab.exclstr) }
  .stab.index 0 : { *(.stab.index) }
  .stab.indexstr 0 : { *(.stab.indexstr) }
  .comment 0 : { *(.comment) }
  /* DWARF debug sections.
     Symbols in the DWARF debugging sections are relative to the beginning
     of the section so we begin them at 0.  */
  /* DWARF 1 */
  .debug          0 : { *(.debug) }
  .line           0 : { *(.line) }
  /* GNU DWARF 1 extensions */
  .debug_srcinfo  0 : { *(.debug_srcinfo) }
  .debug_sfnames  0 : { *(.debug_sfnames) }
  /* DWARF 1.1 and DWARF 2 */
  .debug_aranges  0 : { *(.debug_aranges) }
  .debug_pubnames 0 : { *(.debug_pubnames) }
  /* DWARF 2 */
  .debug_info     0 : { *(.debug_info) *(.gnu.linkonce.wi.*) }
  .debug_abbrev   0 : { *(.debug_abbrev) }
  .debug_line     0 : { *(.debug_line) }
  .debug_frame    0 : { *(.debug_frame) }
  .debug_str      0 : { *(.debug_str) }
  .debug_loc      0 : { *(.debug_loc) }
  .debug_macinfo  0 : { *(.debug_macinfo) }
  
  PROVIDE (__stack = 0x2500) ;
  PROVIDE (__data_start_rom = _etext) ;
  PROVIDE (__data_end_rom   = _etext + SIZEOF (.data)) ;
  PROVIDE (__noinit_start_rom = _etext + SIZEOF (.data)) ;
  PROVIDE (__noinit_end_rom = _etext + SIZEOF (.data) + SIZEOF (.noinit)) ;
  PROVIDE (__subdevice_has_heap = 0) ;
}
