 /*
 * Copyright (c) 2008, Freie Universitt Berlin, Tomasz Naumowicz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Freie Universitt Berlin nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#ifndef COMMANDS_H_
#define COMMANDS_H_

typedef struct {
	uint8_t			executing_node;			// the node where the command has to be executed
	char*			command_name;			// the name of the command
	char*			arguments;				// provided by the API
	uint8_t			arguments_count;		// provided by the API
	char*			response;				// to be filled by the command
	uint8_t			response_length;		// provided by the API, the length of the response buffer
} command_args_t;

typedef void (*fp_command_t) (const command_args_t*);

typedef struct {
	fp_command_t	handler;
	char			commandName[8];
	uint8_t 		flags;
} command_t;

#define COMMAND_SERIAL		BIT0
#define COMMAND_REMOTE		BIT1

#define PID_REMOTECOMMAND_REQUEST	220
#define PID_REMOTECOMMAND_RESPONSE	221

#define COMMAND_RESPONSE_LENGTH 		40

#define COMMAND(cmd_name, cmd_flags, cmd_args)	extern void _cmd_ ## cmd_name ## _handler (const command_args_t* cmd_args ); \
												__attribute__((section(".commands"))) const command_t _cmd_ ## cmd_name ## _registration = { _cmd_ ## cmd_name ## _handler, #cmd_name, cmd_flags }; \
												__attribute__((noinline)) \
												void _cmd_ ## cmd_name ## _handler(const command_args_t* cmd_args)



bool Commands_ProcessSerialInput(char* input);

/*
 * Radio has to be initialized already
 */
void Commands_Init();

#endif /*COMMANDS_H_*/
