/**
 * Copyright (c) 2008, Freie Universitt Berlin, Tomasz Naumowicz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Freie Universitt Berlin nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef LOGGING_H_
#define LOGGING_H_

typedef void(*fp_logHandler_t) (uint8_t, const char*, ...);

#define LOGLEVEL_VERBOSE					BIT1
#define LOGLEVEL_ASSERTS					BIT2
#define LOGLEVEL_CODES_WITH_TEXT			BIT3
#define LOGLEVEL_DEBUG						BIT4
#define LOGLEVEL_NONE						0

#ifndef LOGLEVEL
	#define LOGLEVEL						(LOGLEVEL_VERBOSE | LOGLEVEL_DEBUG | LOGLEVEL_CODES_WITH_TEXT)
#endif

/**
 * sets up the default logging handler, the Logging_HandlerUart
 */
void Logging_Init();

#if (LOGLEVEL > 0)
	/**
	 * default logging handler - uses uart output.
	 */
	void Logging_HandlerUart(uint8_t level, const char *fmt, ...);
	
	extern fp_logHandler_t Logging_ActiveHandler;
#endif

//explained in here: http://www.decompile.com/cpp/faq/file_and_line_error_string.htm
#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)
	
#if (LOGLEVEL & LOGLEVEL_VERBOSE)
	#define LOG(_format, args...)						Logging_ActiveHandler(LOGLEVEL_VERBOSE, "[%u:L] " _format, Configuration.NodeID, ##args)
#else
	#define LOG(...)
#endif
	
#if (LOGLEVEL & LOGLEVEL_DEBUG)
	#define LOGDBG(_format, args...)					Logging_ActiveHandler(LOGLEVEL_VERBOSE, "[%u:D] " _format, Configuration.NodeID, ##args)
#else
	#define LOGDBG(...)
#endif
	
#if (LOGLEVEL & LOGLEVEL_CODES_WITH_TEXT)
	#if (LOGLEVEL & LOGLEVEL_VERBOSE) 
		#define LOG_CODE(_code, _format, args...)		Logging_ActiveHandler(LOGLEVEL_VERBOSE, "[%u:c:" ""TOSTRING(_code)"" "] " _format, Configuration.NodeID, ##args)
	#else
		#if (LOGLEVEL & LOGLEVEL_CODES_WITH_TEXT)
			#define LOG_CODE(_code, _format, args...)	Logging_ActiveHandler(LOGLEVEL_VERBOSE, "[%u:c:" ""TOSTRING(_code)"" "]", Configuration.NodeID)
		#else
			#define LOG_CODE(...)
		#endif
	#endif
#else
	#define LOG_CODE(...)
#endif
	
#if (LOGLEVEL & LOGLEVEL_ASSERTS)

	#define AT __FILE__":"TOSTRING(__LINE__)

	#define ASSERT(input)								if (!(input)) { Logging_ActiveHandler(LOGLEVEL_ASSERTS, "[%u:A] "AT, Configuration.NodeID); } 
#else
	#define ASSERT(...)
#endif
	
#endif /*LOGGING_H_*/
