 /*
 * Copyright (c) 2008, Freie Universitt Berlin, Tomasz Naumowicz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Freie Universitt Berlin nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#ifndef NETWORK_H_
#define NETWORK_H_

/**
 * Frequency Channel IDs start with 0
 */
#define NETWORK_FREQUENCY_CHANNELS_COUNT		8

#define NETWORK_MAX_TRANSMIT_POWER				255

#define NETWORK_MAX_TOTAL_PAYLOAD_SIZE			60

#define NETWORK_PACKET_PAYLOAD_COUNT			2

struct netpacket_send_buffer {
	uint8_t	Size;		///< number of bytes to read
	void*	Buffer;		///< pointer to first byte of data
};

typedef struct  {
	/// optional payload blocks that are sent in consecutive order
	struct netpacket_send_buffer		Buffers[NETWORK_PACKET_PAYLOAD_COUNT];
} network_payload_t;

typedef void (*network_packet_handler_t) (uint8_t,uint8_t*); //from, to, packeteType, network_payload_t
typedef void (*network_send_callback_t) (bool success);

void Network_Init(uint8_t nodeID);

void Network_InitSendArgs(network_payload_t* payload);

//not implemented
//void Network_SendWithACK(uint8_t destination, uint8_t packetType, network_payload_t* payload, network_send_callback_t callback);

bool Network_SendWithoutACK(uint8_t destination, uint8_t packetType, network_payload_t* payload);

void Network_RegisterHandler(uint8_t packetType, network_packet_handler_t packetHandler);

void Network_SetFrequencyChannel(uint8_t channel);

uint8_t Network_GetFrequencyChannel();

void Network_SetTransmitPower(uint8_t txpower);

void Network_SetNodeID(uint8_t nodeID);

// states whether all detected radio packets should be received (with destination != broadcast or nodeid)
void Network_SnifferMode(bool enabled);


#endif /*NETWORK_H_*/
