/**
 * Copyright (c) 2008, Freie Universitt Berlin, Tomasz Naumowicz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Freie Universitt Berlin nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SYSTEM_H_
#define SYSTEM_H_

#include "Types.h"

//this defines have to be available. the USART and Timers will be configured for this value and
//will check presense of this defines
#define __MCLK_4MHZ__
#define __ACLK_32KHZ__
#define __MSP430GCC__

#define SCATTERWEB_API_VERSION 		"1.0.0.0"

#define SET(byte, bit)     ((byte) |= (bit))
#define CLEAR(byte, bit)   ((byte) &= (~bit))

extern void Application_Init(); 
extern void Application_ProcessSystemFlags(); 

#include "../Application/AppConfig.h"

//used during development
#define VERBOSEDEBUGGING

#include <stdarg.h>
#include <msp430x16x.h>
#include <io.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <mspgcc/ringbuffer.h>

#include "Programming.h"
#include "Configuration.h"

//////////////////////////////////////// runtime flag
extern volatile uint8_t System_runtime_flag;

#define FLAG_UART_NEWLINE			BIT0
#define FLAG_SOFTWARE_TIMER			BIT1

/* ADDED BY HUTTA */
//- #define FLAG_USER_6					BIT2
#define FLAG_RADIO_TX					BIT2
//- #define FLAG_USER_5					BIT3
#define FLAG_RADIO_RX					BIT3
/* ADDED BY HUTTA */

#define FLAG_USER_4					BIT4
#define FLAG_USER_3					BIT5
#define FLAG_USER_2					BIT6
#define FLAG_USER_1					BIT7

/* ADDED BY HUTTA */
//- #define FLAGS_USER					(FLAG_USER_1 | FLAG_USER_2 | FLAG_USER_3 | FLAG_USER_4 | FLAG_USER_5 | FLAG_USER_6)
#define FLAGS_USER					(FLAG_USER_1 | FLAG_USER_2 | FLAG_USER_3 | FLAG_USER_4)
/* ADDED BY HUTTA */

int main(void);

// defines the COMMAND macro used to register commands
#include "Commands.h"

#include "MSB430H.h"
#include "USART.h"
#include "Timers.h"
#include "RTC.h"
#include "ScatterWeb.Data.h"
#include "Logging.h"

#include "ScatterWeb.Sd.h"
#include "dosfs.h"
#include "dosfs_helpers.h"


#include "Network.h"

#endif /*SYSTEM_H_*/
