/**
 * Copyright (c) 2008, Freie Universitt Berlin, Tomasz Naumowicz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Freie Universitt Berlin nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef TIMERS_H_
#define TIMERS_H_

#define TICKS_PER_SECOND				8192
#define TICKS_PER_MILISECOND			(8.192)	// actually this is 8.192...


/**
 * Reason to define all of them: the precompiler doesn't execute the expressions.
 */

#define TICKS_SECOND					8192
#define TICKS_2SECONDS					16384
#define TICKS_5SECONDS					40960
#define TICKS_10SECONDS					81920
#define TICKS_30SECONDS					245760
#define TICKS_MINUTE					491520			//(8192*60)
#define	TICKS_2MINUTES					983040			//(8192*120)
#define TICKS_10MINUTES					4915200

#ifndef TIMERS_MAX_TIMERS_COUNT
	#define TIMERS_MAX_TIMERS_COUNT		20
#endif

#ifndef __ACLK_32KHZ__
	#error "The TimerA is configured with the assumption that the ACLK runs with 32kHz!"
#endif

typedef void (*fp_timer_t) (uint16_t);

uint32_t *TICKS();

void Timers_Init();

/**
 * Registers a non-blocking timer. After ticks * 122us the timerHandler is invoked.
 * Returns the ID of the registered timer.
 */
int16_t Timers_Add(fp_timer_t timerHandler, uint32_t ticks, uint16_t data);

bool Timers_Remove(int16_t timerId);

void Timers_ProcessPendingTimerHandler();

/**
 * Blocks the execution of the program for 'ticks' number of ticks
 * It doesn't block interrupts.
 * It's not using any energy saving modes.
 */
void Timers_Block(uint16_t ticks);

//uint32_t* Timers_Ticks();

#endif /*TIMERS_H_*/
